
/* Line 3 of config_scanner.c */

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer libvma_yy_create_buffer
#define yy_delete_buffer libvma_yy_delete_buffer
#define yy_flex_debug libvma_yy_flex_debug
#define yy_init_buffer libvma_yy_init_buffer
#define yy_flush_buffer libvma_yy_flush_buffer
#define yy_load_buffer_state libvma_yy_load_buffer_state
#define yy_switch_to_buffer libvma_yy_switch_to_buffer
#define yyin libvma_yyin
#define yyleng libvma_yyleng
#define yylex libvma_yylex
#define yylineno libvma_yylineno
#define yyout libvma_yyout
#define yyrestart libvma_yyrestart
#define yytext libvma_yytext
#define yywrap libvma_yywrap
#define yyalloc libvma_yyalloc
#define yyrealloc libvma_yyrealloc
#define yyfree libvma_yyfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 39
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE libvma_yyrestart(libvma_yyin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

extern yy_size_t libvma_yyleng;

extern FILE *libvma_yyin, *libvma_yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up libvma_yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up libvma_yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	yy_size_t yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via libvma_yyrestart()), so that the user can continue scanning by
	 * just pointing libvma_yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when libvma_yytext is formed. */
static char yy_hold_char;
static yy_size_t yy_n_chars;		/* number of characters read into yy_ch_buf */
yy_size_t libvma_yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow libvma_yywrap()'s to do buffer switches
 * instead of setting up a fresh libvma_yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void libvma_yyrestart (FILE *input_file  );
void libvma_yy_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE libvma_yy_create_buffer (FILE *file,int size  );
void libvma_yy_delete_buffer (YY_BUFFER_STATE b  );
void libvma_yy_flush_buffer (YY_BUFFER_STATE b  );
void libvma_yypush_buffer_state (YY_BUFFER_STATE new_buffer  );
void libvma_yypop_buffer_state (void );

static void libvma_yyensure_buffer_stack (void );
static void libvma_yy_load_buffer_state (void );
static void libvma_yy_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER libvma_yy_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE libvma_yy_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE libvma_yy_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE libvma_yy_scan_bytes (yyconst char *bytes,yy_size_t len  );

void *libvma_yyalloc (yy_size_t  );
void *libvma_yyrealloc (void *,yy_size_t  );
void libvma_yyfree (void *  );

#define yy_new_buffer libvma_yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        libvma_yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            libvma_yy_create_buffer(libvma_yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        libvma_yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            libvma_yy_create_buffer(libvma_yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *libvma_yyin = (FILE *) 0, *libvma_yyout = (FILE *) 0;

typedef int yy_state_type;

extern int libvma_yylineno;

int libvma_yylineno = 1;

extern char *libvma_yytext;
#define yytext_ptr libvma_yytext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up libvma_yytext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	libvma_yyleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 29
#define YY_END_OF_BUFFER 30
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[473] =
    {   0,
        0,    0,    0,    0,    0,    0,    0,    0,   30,   28,
       27,   25,   26,    5,    5,   28,   28,   28,   28,   28,
       28,   28,   28,   28,   28,   27,    1,   24,   24,    5,
        5,   24,   24,   24,   24,   24,   24,   24,   24,   24,
       24,    1,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    1,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    1,   27,   26,    5,    0,    0,    0,    0,   15,
       18,    0,    0,    0,    0,    0,    0,    0,    0,   27,
        1,    1,   24,   24,    5,   24,   24,   24,   24,   15,

       18,   24,   24,   24,   24,   24,   24,   24,   24,    1,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    1,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    1,    0,    0,    6,    0,
       17,    0,   13,   14,   12,   16,    0,    0,    1,   24,
       24,    6,   24,   17,   24,   13,   14,   12,   16,   24,
       24,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    0,    0,    0,    0,    0,

        0,    0,    0,   24,   24,   24,   24,   24,   24,   24,
       24,    3,    3,    3,    3,    3,    3,    3,    3,    4,
        4,    4,    4,    4,    4,    4,    4,    0,    0,    0,
        0,    0,    0,    0,    0,    0,   11,    0,   24,   24,
       24,   24,   24,   24,   24,   24,   24,   11,   24,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    0,    0,    0,   10,    0,    0,    0,    0,    0,
        0,   24,   24,   24,   10,   24,   24,   24,   24,   24,
       24,    3,    3,    3,    3,    3,    3,    3,    3,    3,

        3,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    0,    0,    0,    0,    0,    0,    0,    0,    9,
       24,   24,   24,   24,   24,   24,   24,   24,    9,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    0,    0,    0,
        0,    0,    0,    0,    0,   24,   24,   24,   24,   24,
       24,   24,   24,    3,    3,    3,    3,    3,    3,    3,
        3,    4,    4,    4,    4,    4,    4,    4,    4,    0,
        0,    8,    0,    0,    0,    0,    0,   24,   24,    8,
       24,   24,   24,   24,   24,    3,    3,    3,    3,    3,

        3,    3,    3,    4,    4,    4,    4,    4,    4,    4,
        4,    0,    0,   19,   20,    0,    0,   21,   24,   24,
       19,   20,   24,   24,   21,    3,    3,    3,    3,    3,
        3,    3,    4,    4,    4,    4,    4,    4,    4,    0,
        7,   23,    0,   24,    7,   23,   24,    3,    3,    3,
        3,    4,    4,    4,    4,    0,   22,   24,   22,    3,
        3,    4,    4,    0,   24,    3,    4,    2,    2,    2,
        2,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    1,    4,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    5,    1,    1,    6,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    8,    1,    9,    1,   10,   11,

       12,   13,   14,    1,   15,    1,    1,   16,   17,   18,
       19,   20,    1,   21,   22,   23,   24,   25,    1,    1,
       26,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[27] =
    {   0,
        1,    2,    3,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[485] =
    {   0,
        0,   25,   29,   54,   58,   83,   87,  112,  548,  549,
      545,  549,    0,  549,  111,  526,  533,  525,  528,  520,
      110,  531,  104,  523,  107,  120,    0,    0,  537,    0,
      121,  518,  525,  517,  520,  512,  114,  523,  119,  515,
      120,  529,    0,  528,    0,  125,  509,  516,  508,  511,
      503,  126,  514,  123,  506,  125,  520,    0,  519,    0,
      137,  500,  507,  499,  502,  494,  137,  505,  128,  497,
      136,  511,  510,    0,  147,  491,  488,  495,  490,  549,
      549,  487,  484,  485,  484,  491,  493,  486,  477,  153,
        0,    0,    0,  497,  153,  478,  475,  482,  477,    0,

        0,  474,  471,  472,  471,  478,  480,  473,  464,  484,
        0,  483,  155,  464,  461,  468,  463,    0,    0,  460,
      457,  458,  457,  464,  466,  459,  450,  470,    0,  469,
      158,  450,  447,  454,  449,    0,    0,  446,  443,  444,
      443,  450,  452,  445,  436,  456,  441,  433,  549,  450,
      549,  438,  445,  444,  549,  549,  435,  439,    0,  433,
      425,    0,  442,    0,  430,  437,  436,    0,    0,  427,
      431,  425,  417,    0,  434,    0,  422,  429,  428,    0,
        0,  419,  423,  417,  409,    0,  426,    0,  414,  421,
      420,    0,    0,  411,  415,  410,  409,  407,  403,  146,

      156,  409,  408,  404,  403,  401,  397,  157,  159,  403,
      402,  398,  397,  395,  391,  160,  162,  397,  396,  392,
      391,  389,  385,  163,  165,  391,  390,  391,  382,  387,
      384,  381,  384,  376,  382,  381,  549,  371,  381,  372,
      377,  374,  371,  374,  366,  372,  371,    0,  361,  371,
      362,  367,  364,  361,  364,  356,  362,  361,    0,  351,
      361,  352,  357,  354,  351,  354,  346,  352,  351,    0,
      341,  352,  351,  334,  549,  343,  336,  338,  345,  336,
      332,  343,  342,  325,    0,  334,  327,  329,  336,  327,
      323,  334,  333,  316,    0,  325,  318,  320,  327,  318,

      314,  325,  324,  307,    0,  316,  309,  311,  318,  309,
      305,  302,  301,  311,  310,  296,  302,  307,  307,  549,
      294,  293,  303,  302,  288,  294,  299,  299,    0,  286,
      285,  295,  294,  280,  286,  291,  291,    0,  278,  277,
      287,  286,  272,  278,  283,  283,    0,  278,  277,  275,
      272,  277,  276,  272,  274,  270,  269,  267,  264,  269,
      268,  264,  266,  262,  261,  259,  256,  261,  260,  256,
      258,  254,  253,  251,  248,  253,  252,  248,  250,  242,
      241,  549,  236,  237,  247,  231,  234,  235,  234,    0,
      229,  230,  240,  224,  227,  228,  227,    0,  222,  223,

      233,  217,  220,  221,  220,    0,  215,  216,  226,  210,
      213,  215,  214,  549,  549,  208,  218,  549,  211,  210,
        0,    0,  204,  214,    0,  207,  206,    0,    0,  200,
      210,    0,  203,  202,    0,    0,  196,  206,    0,  212,
      549,  549,  195,  210,    0,    0,  193,  208,    0,    0,
      191,  206,    0,    0,  189,  194,  549,  193,    0,  192,
        0,  191,    0,  165,  163,  160,  140,  549,    0,    0,
        0,  549,  187,  189,   85,  191,  193,   56,  195,  197,
       27,  199,  201,  203
    } ;

static yyconst flex_int16_t yy_def[485] =
    {   0,
      472,    1,  472,    3,  472,    5,  472,    7,  472,  472,
      472,  472,  473,  472,  472,  472,  472,  472,  472,  472,
      472,  472,  472,  472,  472,  472,  474,  475,  476,  475,
      475,  475,  475,  475,  475,  475,  475,  475,  475,  475,
      475,  477,  478,  479,  478,  478,  478,  478,  478,  478,
      478,  478,  478,  478,  478,  478,  480,  481,  482,  481,
      481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
      481,  483,  472,  473,  472,  472,  472,  472,  472,  472,
      472,  472,  472,  472,  472,  472,  472,  472,  472,  472,
      484,  474,  475,  476,  475,  475,  475,  475,  475,  475,

      475,  475,  475,  475,  475,  475,  475,  475,  475,  477,
      478,  479,  478,  478,  478,  478,  478,  478,  478,  478,
      478,  478,  478,  478,  478,  478,  478,  480,  481,  482,
      481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
      481,  481,  481,  481,  481,  483,  472,  472,  472,  472,
      472,  472,  472,  472,  472,  472,  472,  472,  484,  475,
      475,  475,  475,  475,  475,  475,  475,  475,  475,  475,
      475,  478,  478,  478,  478,  478,  478,  478,  478,  478,
      478,  478,  478,  481,  481,  481,  481,  481,  481,  481,
      481,  481,  481,  481,  481,  472,  472,  472,  472,  472,

      472,  472,  472,  475,  475,  475,  475,  475,  475,  475,
      475,  478,  478,  478,  478,  478,  478,  478,  478,  481,
      481,  481,  481,  481,  481,  481,  481,  472,  472,  472,
      472,  472,  472,  472,  472,  472,  472,  472,  475,  475,
      475,  475,  475,  475,  475,  475,  475,  475,  475,  478,
      478,  478,  478,  478,  478,  478,  478,  478,  478,  478,
      481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
      481,  472,  472,  472,  472,  472,  472,  472,  472,  472,
      472,  475,  475,  475,  475,  475,  475,  475,  475,  475,
      475,  478,  478,  478,  478,  478,  478,  478,  478,  478,

      478,  481,  481,  481,  481,  481,  481,  481,  481,  481,
      481,  472,  472,  472,  472,  472,  472,  472,  472,  472,
      475,  475,  475,  475,  475,  475,  475,  475,  475,  478,
      478,  478,  478,  478,  478,  478,  478,  478,  481,  481,
      481,  481,  481,  481,  481,  481,  481,  472,  472,  472,
      472,  472,  472,  472,  472,  475,  475,  475,  475,  475,
      475,  475,  475,  478,  478,  478,  478,  478,  478,  478,
      478,  481,  481,  481,  481,  481,  481,  481,  481,  472,
      472,  472,  472,  472,  472,  472,  472,  475,  475,  475,
      475,  475,  475,  475,  475,  478,  478,  478,  478,  478,

      478,  478,  478,  481,  481,  481,  481,  481,  481,  481,
      481,  472,  472,  472,  472,  472,  472,  472,  475,  475,
      475,  475,  475,  475,  475,  478,  478,  478,  478,  478,
      478,  478,  481,  481,  481,  481,  481,  481,  481,  472,
      472,  472,  472,  475,  475,  475,  475,  478,  478,  478,
      478,  481,  481,  481,  481,  472,  472,  475,  475,  478,
      478,  481,  481,  472,  475,  478,  481,  472,  475,  478,
      481,    0,  472,  472,  472,  472,  472,  472,  472,  472,
      472,  472,  472,  472
    } ;

static yyconst flex_int16_t yy_nxt[576] =
    {   0,
       10,   11,   12,   13,   10,   14,   15,   10,   16,   10,
       17,   10,   10,   10,   10,   18,   19,   10,   20,   10,
       10,   21,   22,   23,   24,   25,   26,  129,   27,   28,
       11,   12,   29,   28,   30,   31,   28,   32,   28,   33,
       28,   28,   28,   28,   34,   35,   28,   36,   28,   28,
       37,   38,   39,   40,   41,   26,  111,   42,   43,   11,
       12,   44,   43,   45,   46,   43,   47,   43,   48,   43,
       43,   43,   43,   49,   50,   43,   51,   43,   43,   52,
       53,   54,   55,   56,   26,   93,   57,   58,   11,   12,
       59,   58,   60,   61,   58,   62,   58,   63,   58,   58,

       58,   58,   64,   65,   58,   66,   58,   58,   67,   68,
       69,   70,   71,   26,   85,   72,   75,   75,   81,   88,
       82,   90,  101,   91,  102,   86,   95,   95,   89,  105,
      113,  113,  108,  123,  119,   83,  120,  126,  141,  103,
      106,  109,  131,  131,  124,  137,  127,  138,  144,  142,
      471,  121,   75,   75,   90,  232,   91,  145,   95,   95,
      113,  113,  139,  131,  131,  234,  243,  233,  245,  254,
      470,  256,  265,  469,  267,  468,  235,  236,  244,  246,
      247,  255,  257,  258,  266,  268,  269,   74,   74,   92,
       92,   94,   94,  110,  110,  112,  112,  128,  128,  130,

      130,  146,  146,  159,  159,  467,  466,  465,  464,  463,
      462,  461,  460,  459,  458,  457,  456,  455,  454,  453,
      452,  451,  450,  449,  448,  447,  446,  445,  444,  443,
      442,  441,  440,  439,  438,  437,  436,  435,  434,  433,
      432,  431,  430,  429,  428,  427,  426,  425,  424,  423,
      422,  421,  420,  419,  418,  417,  416,  415,  414,  413,
      412,  411,  410,  409,  408,  407,  406,  405,  404,  403,
      402,  401,  400,  399,  398,  397,  396,  395,  394,  393,
      392,  391,  390,  389,  388,  387,  386,  385,  384,  383,
      382,  381,  380,  379,  378,  377,  376,  375,  374,  373,

      372,  371,  370,  369,  368,  367,  366,  365,  364,  363,
      362,  361,  360,  359,  358,  357,  356,  355,  354,  353,
      352,  351,  350,  349,  348,  347,  346,  345,  344,  343,
      342,  341,  340,  339,  338,  337,  336,  335,  334,  333,
      332,  331,  330,  329,  328,  327,  326,  325,  324,  323,
      322,  321,  320,  319,  318,  317,  316,  315,  314,  313,
      312,  311,  310,  309,  308,  307,  306,  305,  304,  303,
      302,  301,  300,  299,  298,  297,  296,  295,  294,  293,
      292,  291,  290,  289,  288,  287,  286,  285,  284,  283,
      282,  281,  280,  279,  278,  277,  276,  275,  274,  273,

      272,  271,  270,  264,  263,  262,  261,  260,  259,  253,
      252,  251,  250,  249,  248,  242,  241,  240,  239,  238,
      237,  231,  230,  229,  228,  227,  226,  225,  224,  223,
      222,  221,  220,  219,  218,  217,  216,  215,  214,  213,
      212,  211,  210,  209,  208,  207,  206,  205,  204,  203,
      202,  201,  200,  199,  198,  197,  196,   92,  195,  194,
      193,  192,  191,  190,  189,  188,  187,  186,  185,  184,
       74,   92,  183,  182,  181,  180,  179,  178,  177,  176,
      175,  174,  173,  172,   74,   92,  171,  170,  169,  168,
      167,  166,  165,  164,  163,  162,  161,  160,   74,  158,

      157,  156,  155,  154,  153,  152,  151,  150,  149,  148,
      147,   73,   92,  143,  140,  136,  135,  134,  133,  132,
       74,   92,  125,  122,  118,  117,  116,  115,  114,   74,
       92,  107,  104,  100,   99,   98,   97,   96,   74,   87,
       84,   80,   79,   78,   77,   76,   73,  472,    9,  472,
      472,  472,  472,  472,  472,  472,  472,  472,  472,  472,
      472,  472,  472,  472,  472,  472,  472,  472,  472,  472,
      472,  472,  472,  472,  472
    } ;

static yyconst flex_int16_t yy_chk[576] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    2,  481,    2,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    4,  478,    4,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    6,  475,    6,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,

        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    8,   23,    8,   15,   15,   21,   25,
       21,   26,   37,   26,   37,   23,   31,   31,   25,   39,
       46,   46,   41,   54,   52,   21,   52,   56,   69,   37,
       39,   41,   61,   61,   54,   67,   56,   67,   71,   69,
      467,   52,   75,   75,   90,  200,   90,   71,   95,   95,
      113,  113,   67,  131,  131,  201,  208,  200,  209,  216,
      466,  217,  224,  465,  225,  464,  201,  201,  208,  209,
      209,  216,  217,  217,  224,  225,  225,  473,  473,  474,
      474,  476,  476,  477,  477,  479,  479,  480,  480,  482,

      482,  483,  483,  484,  484,  462,  460,  458,  456,  455,
      452,  451,  448,  447,  444,  443,  440,  438,  437,  434,
      433,  431,  430,  427,  426,  424,  423,  420,  419,  417,
      416,  413,  412,  411,  410,  409,  408,  407,  405,  404,
      403,  402,  401,  400,  399,  397,  396,  395,  394,  393,
      392,  391,  389,  388,  387,  386,  385,  384,  383,  381,
      380,  379,  378,  377,  376,  375,  374,  373,  372,  371,
      370,  369,  368,  367,  366,  365,  364,  363,  362,  361,
      360,  359,  358,  357,  356,  355,  354,  353,  352,  351,
      350,  349,  348,  346,  345,  344,  343,  342,  341,  340,

      339,  337,  336,  335,  334,  333,  332,  331,  330,  328,
      327,  326,  325,  324,  323,  322,  321,  319,  318,  317,
      316,  315,  314,  313,  312,  311,  310,  309,  308,  307,
      306,  304,  303,  302,  301,  300,  299,  298,  297,  296,
      294,  293,  292,  291,  290,  289,  288,  287,  286,  284,
      283,  282,  281,  280,  279,  278,  277,  276,  274,  273,
      272,  271,  269,  268,  267,  266,  265,  264,  263,  262,
      261,  260,  258,  257,  256,  255,  254,  253,  252,  251,
      250,  249,  247,  246,  245,  244,  243,  242,  241,  240,
      239,  238,  236,  235,  234,  233,  232,  231,  230,  229,

      228,  227,  226,  223,  222,  221,  220,  219,  218,  215,
      214,  213,  212,  211,  210,  207,  206,  205,  204,  203,
      202,  199,  198,  197,  196,  195,  194,  191,  190,  189,
      187,  185,  184,  183,  182,  179,  178,  177,  175,  173,
      172,  171,  170,  167,  166,  165,  163,  161,  160,  158,
      157,  154,  153,  152,  150,  148,  147,  146,  145,  144,
      143,  142,  141,  140,  139,  138,  135,  134,  133,  132,
      130,  128,  127,  126,  125,  124,  123,  122,  121,  120,
      117,  116,  115,  114,  112,  110,  109,  108,  107,  106,
      105,  104,  103,  102,   99,   98,   97,   96,   94,   89,

       88,   87,   86,   85,   84,   83,   82,   79,   78,   77,
       76,   73,   72,   70,   68,   66,   65,   64,   63,   62,
       59,   57,   55,   53,   51,   50,   49,   48,   47,   44,
       42,   40,   38,   36,   35,   34,   33,   32,   29,   24,
       22,   20,   19,   18,   17,   16,   11,    9,  472,  472,
      472,  472,  472,  472,  472,  472,  472,  472,  472,  472,
      472,  472,  472,  472,  472,  472,  472,  472,  472,  472,
      472,  472,  472,  472,  472
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int libvma_yy_flex_debug;
int libvma_yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *libvma_yytext;
/* Line 1 of config_scanner.l */
/*
 * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: ibnl_scanner.ll,v 1.4 2005/02/23 21:08:37 eitan Exp $
 */
/* Line 36 of config_scanner.l */

//#define DEBUG 1

#define yyparse libvma_yyparse
#define libvma_yylex   libvma_yylex
#define yyerror libvma_yyerror
#define yylval  libvma_yylval
#define yychar  libvma_yychar
#define yydebug libvma_yydebug
#define yynerrs libvma_yynerrs

#define libvma_yywrap  libvma_yywrap

#include <string.h>
#include <stdio.h>
#include "config_parser.h"
extern long __vma_config_line_num;
#define YY_NO_INPUT 1

/* Line 812 of config_scanner.c */

#define INITIAL 0
#define CANNAME 1
#define APP_ID_S1 2
#define APP_ID_S2 3

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int libvma_yylex_destroy (void );

int libvma_yyget_debug (void );

void libvma_yyset_debug (int debug_flag  );

YY_EXTRA_TYPE libvma_yyget_extra (void );

void libvma_yyset_extra (YY_EXTRA_TYPE user_defined  );

FILE *libvma_yyget_in (void );

void libvma_yyset_in  (FILE * in_str  );

FILE *libvma_yyget_out (void );

void libvma_yyset_out  (FILE * out_str  );

yy_size_t libvma_yyget_leng (void );

char *libvma_yyget_text (void );

int libvma_yyget_lineno (void );

void libvma_yyset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int libvma_yywrap (void );
#else
extern int libvma_yywrap (void );
#endif
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( libvma_yytext, libvma_yyleng, 1, libvma_yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( libvma_yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( libvma_yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, libvma_yyin))==0 && ferror(libvma_yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(libvma_yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int libvma_yylex (void);

#define YY_DECL int libvma_yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after libvma_yytext and libvma_yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( libvma_yyleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(libvma_yytext[libvma_yyleng - 1] == '\n'); \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! libvma_yyin )
			libvma_yyin = stdin;

		if ( ! libvma_yyout )
			libvma_yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			libvma_yyensure_buffer_stack ();
			if ( yy_buffer_stack ) {
				YY_CURRENT_BUFFER_LVALUE =
					libvma_yy_create_buffer(libvma_yyin,YY_BUF_SIZE );
			}
		}

		libvma_yy_load_buffer_state( );
		}

	{
/* Line 57 of config_scanner.l */


/* Line 1033 of config_scanner.c */

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of libvma_yytext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		/* coverity[var_deref_op] */
		yy_current_state = (yy_start);
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)] ;
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 473 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 549 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
/* Line 59 of config_scanner.l */
{}
	YY_BREAK
case 2:
YY_RULE_SETUP
/* Line 61 of config_scanner.l */
{
	yylval.ival = APP_ID;
#ifdef DEBUG 
	printf("APP_ID\n");
#endif
	BEGIN(APP_ID_S1);
	return APP_ID;
}
	YY_BREAK
case 3:
YY_RULE_SETUP
/* Line 70 of config_scanner.l */
{
	yylval.sval = (char *)malloc(strlen(libvma_yytext) + 1);
	strcpy(yylval.sval, libvma_yytext);
#ifdef DEBUG
	printf("PROGRAM:%s\n",yylval.sval);
#endif
	BEGIN(APP_ID_S2);
	return (PROGRAM);
}
	YY_BREAK
case 4:
YY_RULE_SETUP
/* Line 80 of config_scanner.l */
{
	yylval.sval = (char *)malloc(strlen(libvma_yytext) + 1);
	strcpy(yylval.sval, libvma_yytext);
#ifdef DEBUG
	printf("USER_DEFINED_ID_STR:%s\n",yylval.sval);
#endif
	BEGIN(0);
	return (USER_DEFINED_ID_STR);
}
	YY_BREAK
case 5:
YY_RULE_SETUP
/* Line 90 of config_scanner.l */
{ 
	yylval.ival = atoi(libvma_yytext);
#ifdef DEBUG
	printf("INT:%d\n",yylval.ival);
#endif
	return INT;
}
	YY_BREAK
case 6:
YY_RULE_SETUP
/* Line 98 of config_scanner.l */
{
	yylval.ival = LOG;
#ifdef DEBUG
	printf("LOG\n");
#endif
	return LOG;
}
	YY_BREAK
case 7:
YY_RULE_SETUP
/* Line 106 of config_scanner.l */
{
	yylval.ival = DEST;
#ifdef DEBUG
	printf("DEST\n");
#endif
	return DEST;
}
	YY_BREAK
case 8:
YY_RULE_SETUP
/* Line 114 of config_scanner.l */
{
	yylval.ival = LEVEL;
#ifdef DEBUG
	printf("LEVEL\n");
#endif
	return LEVEL;
}
	YY_BREAK
case 9:
YY_RULE_SETUP
/* Line 122 of config_scanner.l */
{
	yylval.ival = STDERR;
#ifdef DEBUG
	printf("STDERR\n");
#endif
	return STDERR;
}
	YY_BREAK
case 10:
YY_RULE_SETUP
/* Line 130 of config_scanner.l */
{
	yylval.ival = SYSLOG;
#ifdef DEBUG
	printf("SYSLOG\n");
#endif
	return SYSLOG;
}
	YY_BREAK
case 11:
YY_RULE_SETUP
/* Line 138 of config_scanner.l */
{
	yylval.ival = FILENAME;
#ifdef DEBUG
	printf("FILENAME\n");
#endif
	BEGIN(CANNAME);
	return FILENAME;
}
	YY_BREAK
case 12:
YY_RULE_SETUP
/* Line 149 of config_scanner.l */
{
	yylval.ival = USE;
#ifdef DEBUG
	printf("USE\n");
#endif
	return USE;
}
	YY_BREAK
case 13:
YY_RULE_SETUP
/* Line 157 of config_scanner.l */
{
	yylval.ival = TCP;
#ifdef DEBUG
	printf("TCP\n");
#endif
	return TCP;
}
	YY_BREAK
case 14:
YY_RULE_SETUP
/* Line 165 of config_scanner.l */
{
	yylval.ival = UDP;
#ifdef DEBUG
	printf("UDP\n");
#endif
	return UDP;
}
	YY_BREAK
case 15:
YY_RULE_SETUP
/* Line 173 of config_scanner.l */
{
	yylval.ival = OS;
#ifdef DEBUG
	printf("OS\n");
#endif
	return OS;
}
	YY_BREAK
case 16:
YY_RULE_SETUP
/* Line 181 of config_scanner.l */
{
	yylval.ival = VMA;
#ifdef DEBUG
	printf("VMA\n");
#endif
	return VMA;
}
	YY_BREAK
case 17:
YY_RULE_SETUP
/* Line 189 of config_scanner.l */
{
	yylval.ival = SDP;
#ifdef DEBUG
	printf("SDP\n");
#endif
	return SDP;
}
	YY_BREAK
case 18:
YY_RULE_SETUP
/* Line 197 of config_scanner.l */
{
	yylval.ival = SA;
#ifdef DEBUG
	printf("SA\n");
#endif
	return SA;
}
	YY_BREAK
case 19:
YY_RULE_SETUP
/* Line 205 of config_scanner.l */
{
	yylval.ival = TCP_CLIENT;
#ifdef DEBUG
	printf("TCP CLIENT\n");
#endif
	return TCP_CLIENT;
}
	YY_BREAK
case 20:
YY_RULE_SETUP
/* Line 213 of config_scanner.l */
{
	yylval.ival = TCP_SERVER;
#ifdef DEBUG
	printf("TCP SERVER\n");
#endif
	return TCP_SERVER;
}
	YY_BREAK
case 21:
YY_RULE_SETUP
/* Line 221 of config_scanner.l */
{
	yylval.ival = UDP_SENDER;
#ifdef DEBUG
	printf("UDP SENDER\n");
#endif
	return UDP_SENDER;
}
	YY_BREAK
case 22:
YY_RULE_SETUP
/* Line 229 of config_scanner.l */
{
	yylval.ival = UDP_RECEIVER;
#ifdef DEBUG
	printf("UDP RECEIVER\n");
#endif
	return UDP_RECEIVER;
}
	YY_BREAK
case 23:
YY_RULE_SETUP
/* Line 237 of config_scanner.l */
{
	yylval.ival = UDP_CONNECT;
#ifdef DEBUG
	printf("UDP CONNECT\n");
#endif
	return UDP_CONNECT;
}
	YY_BREAK
case 24:
YY_RULE_SETUP
/* Line 245 of config_scanner.l */
{
	yylval.sval = (char *)malloc(strlen(libvma_yytext) + 1);
	strcpy(yylval.sval, libvma_yytext);
#ifdef DEBUG
	printf("NAME:%s\n",yylval.sval);
#endif
	BEGIN(0);
	return (NAME);
}
	YY_BREAK
case 25:
/* rule 25 can match eol */
YY_RULE_SETUP
/* Line 255 of config_scanner.l */
{
	__vma_config_line_num++;
#ifdef DEBUG
	printf("LINE\n");
#endif
	yylval.ival = LINE;
	return(LINE);
}
	YY_BREAK
case 26:
YY_RULE_SETUP
/* Line 264 of config_scanner.l */
{
 	__vma_config_line_num++;
}
	YY_BREAK
case 27:
YY_RULE_SETUP
/* Line 268 of config_scanner.l */
{}
	YY_BREAK
case 28:
YY_RULE_SETUP
/* Line 270 of config_scanner.l */
{
#ifdef DEBUG
	printf("CHAR:%c\n",libvma_yytext[0]);
#endif
	return(libvma_yytext[0]);
}
	YY_BREAK
case 29:
YY_RULE_SETUP
/* Line 277 of config_scanner.l */
ECHO;
	YY_BREAK
/* Line 1397 of config_scanner.c */
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(CANNAME):
case YY_STATE_EOF(APP_ID_S1):
case YY_STATE_EOF(APP_ID_S2):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed libvma_yyin at a new source and called
			 * libvma_yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = libvma_yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( libvma_yywrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * libvma_yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of libvma_yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			yy_size_t num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				yy_size_t new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					libvma_yyrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			libvma_yyrestart(libvma_yyin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) libvma_yyrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 473 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 473 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 472);

		return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			yy_size_t offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					libvma_yyrestart(libvma_yyin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( libvma_yywrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve libvma_yytext */
	(yy_hold_char) = *++(yy_c_buf_p);

	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void libvma_yyrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        libvma_yyensure_buffer_stack ();
		if ( yy_buffer_stack ) {
			YY_CURRENT_BUFFER_LVALUE =
            			libvma_yy_create_buffer(libvma_yyin,YY_BUF_SIZE );
		}
	}

	libvma_yy_init_buffer(YY_CURRENT_BUFFER,input_file );
	libvma_yy_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void libvma_yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		libvma_yypop_buffer_state();
	 *		libvma_yypush_buffer_state(new_buffer);
     */
	libvma_yyensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( yy_buffer_stack ) {
		YY_CURRENT_BUFFER_LVALUE = new_buffer;
	}
	libvma_yy_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (libvma_yywrap()) processing, but the only time this flag
	 * is looked at is after libvma_yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void libvma_yy_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	libvma_yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE libvma_yy_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) libvma_yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in libvma_yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) libvma_yyalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in libvma_yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	libvma_yy_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with libvma_yy_create_buffer()
 * 
 */
    void libvma_yy_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		libvma_yyfree((void *) b->yy_ch_buf  );

	libvma_yyfree((void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a libvma_yyrestart() or at EOF.
 */
    static void libvma_yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;

	if (!b) return;

	libvma_yy_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then libvma_yy_init_buffer was _probably_
     * called from libvma_yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void libvma_yy_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		libvma_yy_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void libvma_yypush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	libvma_yyensure_buffer_stack();

	/* This block is copied from libvma_yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER) {
		(yy_buffer_stack_top)++;
	}
	if (yy_buffer_stack) {
		YY_CURRENT_BUFFER_LVALUE = new_buffer;
	}

	/* copied from libvma_yy_switch_to_buffer. */
	libvma_yy_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void libvma_yypop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	libvma_yy_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		libvma_yy_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void libvma_yyensure_buffer_stack (void)
{
	yy_size_t num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)libvma_yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in libvma_yyensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)libvma_yyrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in libvma_yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE libvma_yy_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) libvma_yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in libvma_yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	libvma_yy_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to libvma_yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       libvma_yy_scan_bytes() instead.
 */
YY_BUFFER_STATE libvma_yy_scan_string (yyconst char * yystr )
{
    
	return libvma_yy_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to libvma_yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE libvma_yy_scan_bytes  (yyconst char * yybytes, yy_size_t  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	yy_size_t i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) libvma_yyalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in libvma_yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = libvma_yy_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in libvma_yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up libvma_yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		libvma_yytext[libvma_yyleng] = (yy_hold_char); \
		(yy_c_buf_p) = libvma_yytext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		libvma_yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int libvma_yyget_lineno  (void)
{
        
    return libvma_yylineno;
}

/** Get the input stream.
 * 
 */
FILE *libvma_yyget_in  (void)
{
        return libvma_yyin;
}

/** Get the output stream.
 * 
 */
FILE *libvma_yyget_out  (void)
{
        return libvma_yyout;
}

/** Get the length of the current token.
 * 
 */
yy_size_t libvma_yyget_leng  (void)
{
        return libvma_yyleng;
}

/** Get the current token.
 * 
 */

char *libvma_yyget_text  (void)
{
        return libvma_yytext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void libvma_yyset_lineno (int  line_number )
{
    
    libvma_yylineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see libvma_yy_switch_to_buffer
 */
void libvma_yyset_in (FILE *  in_str )
{
        libvma_yyin = in_str ;
}

void libvma_yyset_out (FILE *  out_str )
{
        libvma_yyout = out_str ;
}

int libvma_yyget_debug  (void)
{
        return libvma_yy_flex_debug;
}

void libvma_yyset_debug (int  bdebug )
{
        libvma_yy_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from libvma_yylex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    libvma_yyin = stdin;
    libvma_yyout = stdout;
#else
    libvma_yyin = (FILE *) 0;
    libvma_yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * libvma_yylex_init()
     */
    return 0;
}

/* libvma_yylex_destroy is for both reentrant and non-reentrant scanners. */
int libvma_yylex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		libvma_yy_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		libvma_yypop_buffer_state();
	}

	/* Destroy the stack itself. */
	libvma_yyfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * libvma_yylex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *libvma_yyalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *libvma_yyrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void libvma_yyfree (void * ptr )
{
	free( (char *) ptr );	/* see libvma_yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

/* Line 276 of config_scanner.l */



int libvma_yywrap ()
{
	return (1);
}


